Files
TheRock/rocm-systems/projects/hip-tests/catch/unit/memory/hipMemPrefetchAsyncExtTsts.cc
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

295 lines
10 KiB
C++

/*
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
*
* SPDX-License-Identifier: MIT
*/
/* Test Case Description:
1) Allocate managed memory --> prefetch to gpu 0
call hipMemAdvise() on the memory and apply the flags ReadMostly,
AccessedBy, and PreferredLocation for gpus other than gpu 0 and verify
the flags using hipMemGetAttribute()
2) Allocate managed memory --> set AccessedBy using
hipMemAdvise() to gpu1 prefetch the memory to gpu 0 and then query for
AccessedBy using hipMemGetAttribute() and validate if AccessedBy is still
set to gpu1. Similar tests are done with ReadMostly and PreferredLocation
flags
3) Negative testing with hipMemPrefetchAsync() api
4) In this test case I am trying to allocate HMM memory
which is not multiple of page Size, but still trying to launch kernel and
see if we are getting values as expected.
*/
#include <hip_test_common.hh>
// Kernel function
__global__ void MemPrftchAsyncKernel1(int* Hmm, size_t N) {
size_t offset = (blockIdx.x * blockDim.x + threadIdx.x);
size_t stride = blockDim.x * gridDim.x;
for (size_t i = offset; i < N; i += stride) {
Hmm[i] = Hmm[i] * Hmm[i];
}
}
/* Test Case Description: Allocate managed memory --> prefetch to gpu 0
call hipMemAdvise() on the memory and apply the flags ReadMostly,
AccessedBy, and PreferredLocation for gpus other than gpu 0 and verify
the flags using hipMemGetAttribute()*/
HIP_TEST_CASE(Unit_hipMemPrefetchAsyncAdviseFlgTst) {
CHECK_MANAGED_MEMORY_SUPPORT
int NGpus = 0;
HIP_CHECK(hipGetDeviceCount(&NGpus));
if (NGpus < 2) {
HIP_SKIP_TEST(HipTest::SkipReason::kFewerThanTwoGpus);
}
int *Hmm = nullptr, MemSz = (4096 * 4), InitVal = 123;
int Outpt = 9999, NumElms = MemSz / 4;
bool IfTestPassed = true;
hipStream_t strm;
HIP_CHECK(hipStreamCreate(&strm));
HIP_CHECK(hipMallocManaged(&Hmm, MemSz));
// Initializing the memory
for (int i = 0; i < NumElms; ++i) {
Hmm[i] = InitVal;
}
HIP_CHECK(hipMemPrefetchAsync(Hmm, MemSz, 0, strm));
HIP_CHECK(hipStreamSynchronize(strm));
HIP_CHECK(hipMemAdvise(Hmm, MemSz, hipMemAdviseSetReadMostly, 1));
HIP_CHECK(
hipMemRangeGetAttribute(&Outpt, sizeof(int), hipMemRangeAttributeReadMostly, Hmm, MemSz));
if (Outpt != 1) {
WARN(
"hipMemRangeAttributeReadMostly flag did not take effect"
" as expected!!");
IfTestPassed = false;
}
HIP_CHECK(hipMemAdvise(Hmm, MemSz, hipMemAdviseSetAccessedBy, 1));
HIP_CHECK(
hipMemRangeGetAttribute(&Outpt, sizeof(int), hipMemRangeAttributeAccessedBy, Hmm, MemSz));
if (Outpt != 1) {
WARN(
"hipMemRangeAttributeAccessedBy flag did not take effect"
" as expected!!");
IfTestPassed = false;
}
HIP_CHECK(hipMemAdvise(Hmm, MemSz, hipMemAdviseSetPreferredLocation, 1));
HIP_CHECK(hipMemRangeGetAttribute(&Outpt, sizeof(int), hipMemRangeAttributePreferredLocation,
Hmm, MemSz));
if (Outpt != 1) {
WARN(
"hipMemRangeAttributePreferredLocation flag did not take effect"
" as expected!!");
IfTestPassed = false;
}
HIP_CHECK(hipStreamDestroy(strm));
HIP_CHECK(hipFree(Hmm));
REQUIRE(IfTestPassed);
}
/* Test Case description: Allocate managed memory --> set AccessedBy using
hipMemAdvise() to gpu1 prefetch the memory to gpu 0 and then query for
AccessedBy using hipMemGetAttribute() and validate if AccessedBy is still
set to gpu1. Similar tests are done with ReadMostly and PreferredLocation
flags */
HIP_TEST_CASE(Unit_hipMemPrefetchAsyncAccsdByTst) {
CHECK_MANAGED_MEMORY_SUPPORT
int NGpus = 0;
HIP_CHECK(hipGetDeviceCount(&NGpus));
if (NGpus < 2) {
HIP_SKIP_TEST(HipTest::SkipReason::kFewerThanTwoGpus);
}
int *Hmm = nullptr, MemSz = (4096 * 4), InitVal = 123, NumElms = MemSz / 4;
int Outpt = 9999;
bool IfTestPassed = true;
hipStream_t strm;
HIP_CHECK(hipStreamCreate(&strm));
HIP_CHECK(hipMallocManaged(&Hmm, MemSz));
// Initializing the memory
for (int i = 0; i < NumElms; ++i) {
Hmm[i] = InitVal;
}
SECTION("Test AccessedBy with Prefetch") {
HIP_CHECK(hipMemAdvise(Hmm, MemSz, hipMemAdviseSetAccessedBy, 1));
HIP_CHECK(hipMemPrefetchAsync(Hmm, MemSz, 0, strm));
HIP_CHECK(hipStreamSynchronize(strm));
HIP_CHECK(hipMemRangeGetAttribute(&Outpt, sizeof(int), hipMemRangeAttributeAccessedBy, Hmm,
MemSz));
if (Outpt != 1) {
WARN(
"hipMemRangeAttributeAccessedBy flag did not take effect"
" as expected!!");
IfTestPassed = false;
}
}
SECTION("Test ReadMostly with Prefetch") {
HIP_CHECK(hipMemAdvise(Hmm, MemSz, hipMemAdviseSetReadMostly, 1));
HIP_CHECK(hipMemPrefetchAsync(Hmm, MemSz, 0, strm));
HIP_CHECK(hipStreamSynchronize(strm));
MemPrftchAsyncKernel1<<<(NumElms / 32), 32, 0, strm>>>(Hmm, NumElms);
HIP_CHECK(hipStreamSynchronize(strm));
HIP_CHECK(hipMemRangeGetAttribute(&Outpt, sizeof(int), hipMemRangeAttributeReadMostly, Hmm,
MemSz));
if (Outpt != 1) {
WARN(
"hipMemRangeAttributeReadMostly flag did not take effect"
" as expected!!");
IfTestPassed = false;
}
// Verifying the results
for (int i = 0; i < NumElms; ++i) {
if (Hmm[i] != (InitVal * InitVal)) {
WARN("Did not receive expected value!!");
IfTestPassed = false;
break;
}
}
}
SECTION("Test PreferredLocation with Prefetch") {
HIP_CHECK(hipMemAdvise(Hmm, MemSz, hipMemAdviseSetPreferredLocation, 1));
HIP_CHECK(hipMemPrefetchAsync(Hmm, MemSz, 0, strm));
HIP_CHECK(hipStreamSynchronize(strm));
MemPrftchAsyncKernel1<<<(NumElms / 32), 32, 0, strm>>>(Hmm, NumElms);
HIP_CHECK(hipStreamSynchronize(strm));
HIP_CHECK(hipMemRangeGetAttribute(&Outpt, sizeof(int),
hipMemRangeAttributePreferredLocation, Hmm, MemSz));
if (Outpt != 1) {
WARN(
"hipMemRangeAttributePreferredLocation flag did not take effect"
" as expected!!");
IfTestPassed = false;
}
// Verifying the results
for (int i = 0; i < NumElms; ++i) {
if (Hmm[i] != (InitVal * InitVal)) {
WARN("Did not receive expected value!!");
IfTestPassed = false;
break;
}
}
}
HIP_CHECK(hipFree(Hmm));
HIP_CHECK(hipStreamDestroy(strm));
REQUIRE(IfTestPassed);
}
/*Test Case description: Negative testing with hipMemPrefetchAsync() api*/
HIP_TEST_CASE(Unit_hipMemPrefetchAsyncNegativeTst) {
CHECK_MANAGED_MEMORY_SUPPORT
int *Hmm = nullptr, MemSz = 4096 * 4, NumElms = MemSz / 4, InitVal = 123;
bool IfTestPassed = true;
HIP_CHECK(hipMallocManaged(&Hmm, MemSz));
for (int i = 0; i < NumElms; ++i) {
Hmm[i] = InitVal;
}
hipError_t err;
hipStream_t strm;
HIP_CHECK(hipStreamCreate(&strm));
SECTION("Passing null for dev ptr") {
err = hipMemPrefetchAsync(NULL, MemSz, 0, strm);
if (err == hipSuccess) {
WARN("hipMemPrefetchAsync() gives hipSuccess when NULL is passed!!");
IfTestPassed = false;
}
}
SECTION("Passing unusual count size(2nd param)") {
// Passing count size as zero
// expectation: Api should return error
err = hipMemPrefetchAsync(Hmm, 0, 0, strm);
if (err == hipSuccess) {
WARN(
"hipMemPrefetchAsync() gives hipSuccess when count size is"
" passed as zero!!");
IfTestPassed = false;
}
// Passing count size half of actually allocated
// expectation: No issue should be observed
err = hipMemPrefetchAsync(Hmm, MemSz / 2, 0, strm);
if (err != hipSuccess) {
WARN(
"hipMemPrefetchAsync() returned error when count size passed is"
" half of actually allocated!!");
IfTestPassed = false;
}
// Passing count size double that of actually allocated
// expectation: Api should return error
err = hipMemPrefetchAsync(Hmm, MemSz * 2, 0, strm);
if (err == hipSuccess) {
WARN(
"hipMemPrefetchAsync() gives hipSuccess when count size passed is"
" double that of actually allocated!!");
IfTestPassed = false;
}
}
SECTION("Passing invalid device Ordinal") {
err = hipMemPrefetchAsync(Hmm, MemSz, 9999, strm);
if (err == hipSuccess) {
WARN(
"hipMemPrefetchAsync() gives hipSuccess when Invalid device"
" ordinal is passed!!");
IfTestPassed = false;
}
}
SECTION("Checking behavior with stream object") {
// Passing Null stream
// expectation: No issue should be observed
err = hipMemPrefetchAsync(Hmm, MemSz, 0, 0);
if (err != hipSuccess) {
WARN(
"hipMemPrefetchAsync() returns error when Null stream is"
"passed!!");
IfTestPassed = false;
}
// Passing stream object belong to destination device
// expectation: No issue should be observed
int NGpus = 0;
HIP_CHECK(hipGetDeviceCount(&NGpus));
if (NGpus > 1) {
hipStream_t strm1;
HIP_CHECK(hipSetDevice(1));
HIP_CHECK(hipStreamCreate(&strm1));
err = hipMemPrefetchAsync(Hmm, MemSz, 1, strm1);
if (err != hipSuccess) {
WARN(
"hipMemPrefetchAsync() returns error when stream object"
" created in the context of destination gpu is passed!!");
IfTestPassed = false;
}
HIP_CHECK(hipStreamDestroy(strm1));
}
}
HIP_CHECK(hipFree(Hmm));
HIP_CHECK(hipStreamDestroy(strm));
REQUIRE(IfTestPassed);
}
/* Test Case description: In this test case I am trying to allocate HMM memory
which is not multiple of page Size, but still trying to launch kernel and
see if we are getting values as expected.*/
HIP_TEST_CASE(Unit_hipMemPrefetchAsync_NonPageSz) {
int *Hmm = nullptr, NumElms = 4096 * 2, InitVal = 123;
hipStream_t strm;
bool IfTestPassed = true;
HIP_CHECK(hipStreamCreate(&strm));
// Allocating memory = 2*Page Size + 8 bytes
HIP_CHECK(hipMallocManaged(&Hmm, (NumElms * sizeof(int) + 8)));
for (int i = 0; i < (NumElms + 2); ++i) {
Hmm[i] = InitVal;
}
HIP_CHECK(hipMemPrefetchAsync(Hmm, (NumElms * sizeof(int) + 8), 0, strm));
HIP_CHECK(hipStreamSynchronize(strm));
MemPrftchAsyncKernel1<<<((NumElms + 2) / 32 + 1), 32, 0, strm>>>(Hmm, (NumElms + 2));
HIP_CHECK(hipStreamSynchronize(strm));
for (int i = 0; i < (NumElms + 2); ++i) {
if (Hmm[i] != (InitVal * InitVal)) {
WARN("Didnt receive expected output after kernel launch!!");
IfTestPassed = false;
break;
}
}
HIP_CHECK(hipFree(Hmm));
HIP_CHECK(hipStreamDestroy(strm));
REQUIRE(IfTestPassed);
}